# -*- coding: utf-8 -*-
"""server module"""

__author__  = "Adrien DELLE CAVE"
__license__ = """
    Copyright (C) 2018  fjord-technologies

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import logging
import re

from dwho.classes.modules import DWhoModuleBase, MODULES
from httpdis.ext.httpdis_json import HttpReqErrJson
from sonicprobe.libs import xys
from sonicprobe.libs.moresynchro import RWLock


LOG = logging.getLogger('xtrargon.modules.server')


xys.add_regex('server.client_id', re.compile('^[a-zA-Z0-9\.\-]+$').match)


class XtrargonServerModule(DWhoModuleBase):
    MODULE_NAME = 'server'

    LOCK        = RWLock()

    def safe_init(self, options):
        self.server_id     = self.config['general']['server_id']
        self.lock_timeout  = self.config['general']['lock_timeout']

    def _build_client_config(self, client_id):
        r = {'redis': {},
             'stream':  {}}

        if self.config['general'].get('redis'):
            r['redis'] = self.config['general']['redis']

        if 'plugins' in self.config \
           and 'stream' in self.config['plugins']:
            r['stream']['listen_addr'] = self.config['plugins']['stream']['listen_addr']
            r['stream']['listen_port'] = self.config['plugins']['stream']['listen_port']

        if 'client' in self.config \
           and client_id in self.config['client']:
            r.update(self.config['client'])

        return r


    CLIENT_CONFIG_SCHEMA = xys.load("""
    client_id!-: !~~regex(server.client_id)
    """)

    def client_config(self, request):
        params = request.query_params()

        if not xys.validate(params, self.CLIENT_CONFIG_SCHEMA):
            raise HttpReqErrJson(415, "invalid arguments for command")

        if not self.LOCK.acquire_read(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for writing after %s seconds" % self.lock_timeout)

        try:
            return self._build_client_config(params['client_id'])
        except HttpReqErrJson:
            raise
        except Exception, e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()


if __name__ != "__main__":
    def _start():
        MODULES.register(XtrargonServerModule())
    _start()
